package mastermind.model;

import java.io.Serializable;

/**
 * Mastermind Settings: Hilfsobjekt für die Optionen
 */
/* -----------------------------------------------------------------------------
 * Version History:
 * 2012-05-03.v01   FeRo    init version
 * 2012-05-05.v01   FeRo    Design Anpassungen / SettingsHandler
 * 2012-05-18.v01   FeRo    Serializable
 * 2012-05-19.v01   FeRo    Design added
 * -----------------------------------------------------------------------------
 * @author Felix Rohrer <felix.rohrer@stud.hslu.ch>
 * @version 2012-05-19.v01
 */
public class Settings implements Serializable
{
    // Min, Max Values

    /**
     * Minimale Anzahl Farben
     */
    public static final int ANZ_COLOR_MIN = 4; // 4
    /**
     * Maximale Anzahl Farben
     */
    public static final int ANZ_COLOR_MAX = 8; // 8
    /**
     * Minimale Anzahl Steckplätze
     */
    public static final int ANZ_PEG_MIN = 4; // 4
    /**
     * Maximale Anzahl Steckplätze
     */
    public static final int ANZ_PEG_MAX = 10; // 10
    /**
     * Minimale Anzahl Rateversuchen
     */
    public static final int ANZ_GUESS_MIN = 6; // 6
    /**
     * Maximale Anzahl Rateversuchen
     */
    public static final int ANZ_GUESS_MAX = 12; // 12
    /**
     * Maximale Anzahl Design
     */
    public static final int ANZ_DESIGN_MAX = 7; // 7
    // Settings Values
    private int numberOfColors; // Anzahl Farben
    private int numberOfPegs; // Anzahl Steckplätze pro Zeile
    private boolean allowDuplicateColor; // Doppelte Farben erlaubt
    private int numberOfRows; // Anzahl Zeilen
    private int idDesignUsed; // Verwendetes Design

    /**
     * Default Konstruktor
     */
    public Settings()
    {
        this.numberOfColors = ANZ_COLOR_MIN;
        this.numberOfPegs = ANZ_PEG_MIN;
        this.allowDuplicateColor = true;
        this.numberOfRows = ANZ_GUESS_MIN;
        this.idDesignUsed = 2;
    }

    /**
     * Set Mastermind Settings.
     * @param numberOfColors Anzahl Farben
     * @param allowDuplicateColor Doppelte Farben erlaubt
     * @param numberOfPegs Anzahl Steckplätze
     * @param numberOfRows Anzahl Zeilen zum Raten
     * @throws Exception IllegalArgumentException
     */
    public void setSettings(int numberOfColors, boolean allowDuplicateColor, int numberOfPegs, int numberOfRows) throws Exception
    {
        /*
        System.out.println("setSettings()");
        System.out.println("  numberOfColors: " + numberOfColors);
        System.out.println("  duplicateColor: " + allowDuplicateColor);
        System.out.println("  numberOfPegs:   " + numberOfPegs);
        System.out.println("  numberOfRows:   " + numberOfRows);
         */
        this.setNumberOfColors(numberOfColors);
        this.setAllowDuplicateColor(allowDuplicateColor);
        this.setNumberOfPegs(numberOfPegs);
        this.setNumberOfRows(numberOfRows);
    }

    /**
     * Gibt die Anzahl mögliche Farben zurück
     * @return the numberOfColors
     */
    public int getNumberOfColors()
    {
        return numberOfColors;
    }

    /**
     * Setzt die Anzahl der möglichen Farben
     * @param numberOfColors the numberOfColors to set
     * @throws Exception IllegalArgumentException
     */
    public void setNumberOfColors(int numberOfColors) throws Exception
    {
        if ((numberOfColors < ANZ_COLOR_MIN) || (numberOfColors > ANZ_COLOR_MAX)) {
            this.numberOfColors = ANZ_COLOR_MIN;
            throw new IllegalArgumentException("NumberOfColor: out of range: [" + ANZ_COLOR_MIN + ".." + ANZ_COLOR_MAX + "]");
        } else {
            this.numberOfColors = numberOfColors;
        }
    }

    /**
     * Gibt die Anzahl Steckplätze zurück
     * @return the numberOfPegs
     */
    public int getNumberOfPegs()
    {
        return numberOfPegs;
    }

    /**
     * Setzt die Anzahl Steckplätze
     * @param numberOfPegs the numberOfPegs to set
     * @throws Exception IllegalArgumentException
     */
    public void setNumberOfPegs(int numberOfPegs) throws Exception
    {
        if ((numberOfPegs < ANZ_PEG_MIN) || (numberOfPegs > ANZ_PEG_MAX)) {
            this.numberOfPegs = ANZ_PEG_MIN;
            throw new IllegalArgumentException("NumberOfPeg: out of range: [" + ANZ_PEG_MIN + ".." + ANZ_PEG_MAX + "]");
        } else {
            this.numberOfPegs = numberOfPegs;
        }
    }

    /**
     * Gibt zurück ob doppelte Farben erlaubt sind
     * @return allowDuplicateColor
     */
    public boolean getAllowDuplicateColor()
    {
        return allowDuplicateColor;
    }

    /**
     * Erlaubt / sperrt die Option doppelte Farben
     * @param allowDuplicateColor the allowDuplicateColor to set
     */
    public void setAllowDuplicateColor(boolean allowDuplicateColor)
    {
        this.allowDuplicateColor = allowDuplicateColor;
    }

    /**
     * Gibt die Anzahl Zeilen zurück
     * @return the numberOfRows
     */
    public int getNumberOfRows()
    {
        return numberOfRows;
    }

    /**
     * Setzt die Anzahl Zeilen
     * @param numberOfRows the numberOfRows to set
     * @throws Exception IllegalArgumentException
     */
    public void setNumberOfRows(int numberOfRows) throws Exception
    {
        if ((numberOfRows < ANZ_GUESS_MIN) || (numberOfRows > ANZ_GUESS_MAX)) {
            this.numberOfRows = ANZ_GUESS_MIN;
            throw new IllegalArgumentException("NumberOfRow: out of range: [" + ANZ_GUESS_MIN + ".." + ANZ_GUESS_MAX + "]");
        } else {
            this.numberOfRows = numberOfRows;
        }
    }

    /**
     * Gibt die ID des verwendete Design zurück
     * @return ID des Design
     */
    public int getDesign()
    {
        return idDesignUsed;
    }

    /**
     * Setzt das zu verwendende Design
     * @param idDesign ID des zu verwendenden Design
     */
    public void setDesign(int idDesign)
    {
        if (idDesign > ANZ_DESIGN_MAX) {
            this.idDesignUsed = 1;
        } else {
            this.idDesignUsed = idDesign;
        }
    }
}
