package mastermind.solver;

import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;
import mastermind.logic.SettingsHandler;
import mastermind.model.Guess;
import mastermind.model.Result;
import mastermind.model.Settings;

/**
 * Mastermind Solver: Mögliche Lösungen ausfiltern bis nur noch eine übrig ist.
 */
/* -----------------------------------------------------------------------------
 * Version History:
 * 2012-05-06.v01   FeRo    init version
 * 2012-05-24.v01   FeRo    Kombinationen ohne Wiederholungen impl.
 * 2012-05-25.v01   FeRo    MultiThreading für Solver Implementiert
 * 2012-05-25.v02   FeRo    FirstGuess ausgelagert, computeAllPossibleGuess
 * 2012-05-30.v01   FeRo    Alternative Solving Logik getestet, Code CleanUp
 * -----------------------------------------------------------------------------
 * @author Felix Rohrer <felix.rohrer@stud.hslu.ch>
 * @version 2012-05-30.v01
 */
public class SolverLogik implements ISolver
{

    private List<Guess> possibleGuess, possibleGuessResult;
    private int numberOfColors = Settings.ANZ_COLOR_MIN;
    private boolean allowDuplicateColor = false;
    private int numberOfPegs = Settings.ANZ_PEG_MIN;
    private Guess lastGuess;
    private Result lastResult;
    private int nbrOfThreads;

    /**
     * Default Konstruktor
     * @param firstGuess 1. Guess
     * @param firstResult 1. Resultat
     */
    public SolverLogik(Guess firstGuess, Result firstResult)
    {
        // Enum Number of Processors -> creat for each Processor one Thread
        Runtime runtime = Runtime.getRuntime();
        nbrOfThreads = runtime.availableProcessors();
        //System.out.println("# of Processors: " + nbrOfThreads);

        // init Solver
        this.lastGuess = firstGuess;
        this.lastResult = firstResult;
        this.numberOfColors = SettingsHandler.getInstance().getSettings().getNumberOfColors();
        this.allowDuplicateColor = SettingsHandler.getInstance().getSettings().getAllowDuplicateColor();
        this.numberOfPegs = SettingsHandler.getInstance().getSettings().getNumberOfPegs();

        //System.out.println("Init computeAllPossibleGuess...");
        computeAllPossibleGuess(firstGuess, firstResult);
    }

    /**
     * Berechnet alle möglichen Kombinationen
     * @param firstGuess 1. Guess
     * @param firstResult 1. Resultat
     */
    @SuppressWarnings("unchecked")
    public void computeAllPossibleGuess(Guess firstGuess, Result firstResult)
    {
        long startTime, endTime;
        startTime = System.currentTimeMillis();

        int maxGuess;

        if (allowDuplicateColor) {
            maxGuess = (int) Math.pow(numberOfColors, numberOfPegs);
            //System.out.println("allowDuplicateColor: Yes => " + maxGuess + " Perm. (ArraySize: " + (100 / Integer.MAX_VALUE) * maxGuess + "%)");
        } else {
            maxGuess = (factorial(numberOfPegs) / factorial(numberOfPegs - numberOfColors));
            //System.out.println("allowDuplicateColor: No => " + maxGuess + " Perm. (ArraySize: " + (100 / Integer.MAX_VALUE) * maxGuess + "%)");
        }
        if (maxGuess > 1000000000) {
            System.out.println("too many permutations - sorry...");
            System.exit(-1);
        }
        System.out.println("Possible Solutions: " + NumberFormat.getInstance().format(maxGuess));

        possibleGuess = new ArrayList<Guess>(maxGuess);

        Guess newGuess;
        if (allowDuplicateColor) {
            // duplicate color

            Thread[] threads;
            int startPos, endPos;
            int nbrOfElementsPerSubList = (int) Math.floor(maxGuess / nbrOfThreads);
            threads = new Thread[nbrOfThreads];

            // Thread inizialisieren und starten
            for (int i = 0; i < nbrOfThreads; i ++) {
                // Start und End Pos in der Liste berechnen für die einzelnen Threads
                startPos = i * nbrOfElementsPerSubList;
                if (i < (nbrOfThreads - 1)) {
                    endPos = ((i + 1) * nbrOfElementsPerSubList) - 1;
                } else {
                    endPos = maxGuess;
                }
                //System.out.println(i + ": " + startPos + " - " + endPos);
                SolverThreadGenAllGuess myThread = new SolverThreadGenAllGuess(this, startPos, endPos, firstGuess, firstResult, i);
                threads[i] = new Thread(myThread);
                threads[i].start();
            }

            // Warten bis alle Threads fertig sind
            try {
                for (int i = 0; i < nbrOfThreads; i ++) {
                    threads[i].join();
                }
            } catch (InterruptedException ex) {
                System.out.println("Error: " + ex.getLocalizedMessage());
            }

        } else {
            // no duplicate color
            PermutationGenerator<Integer> permutations;
            List<Integer> elements = new ArrayList<Integer>();
            for (int i = 1; i <= numberOfColors; i ++) {
                elements.add(i);
            }
            // generate all combinations
            int moveIndex = 0;
            CombinationGenerator<Integer> combinations = new CombinationGenerator(elements, numberOfPegs);
            for (List<Integer> c : combinations) {
                // generate all permutations
                permutations = new PermutationGenerator(c);
                for (List<Integer> p : permutations) {
                    moveIndex ++;
                    int[] tmpGuess = new int[numberOfPegs];
                    for (int i = 0; i < numberOfPegs; i ++) {
                        tmpGuess[i] = p.get(i);
                    }
                    newGuess = new Guess(tmpGuess);
                    // System.out.println(moveIndex + ": " + newGuess);
                    // Is this Guess a possible Guess for the current Patten?
                    Result result = firstGuess.compareGuess(newGuess);
                    if (result.equals(firstResult)) {
                        // same result, this guess could be the solution
                        possibleGuess.add(newGuess);
                    }
                }
            }
        }

        endTime = System.currentTimeMillis();
        Long lngExecTime = endTime - startTime;
        String strExecTime = String.format("%d min, %d sec",
            TimeUnit.MILLISECONDS.toMinutes(lngExecTime),
            TimeUnit.MILLISECONDS.toSeconds(lngExecTime) -
            TimeUnit.MINUTES.toSeconds(TimeUnit.MILLISECONDS.toMinutes(lngExecTime)));
        System.out.println("computeAllPossibleGuess done... (" + strExecTime + ")");
        //System.out.println("Anz possible Guess: " + NumberFormat.getInstance().format(possibleGuess.size()));
    }

    /**
     * Berechnet die Fakultät einer Zahl
     * @param n Zahl der zu berechnenden Fakultät
     * @return Fakultät
     */
    private static int factorial(int n)
    {
        int res = 1;
        for (int i = n; i >=
            1; i --) {
            res *= i;
        }
        return res;
    }

    /**
     * Liefert den nächsten Rateversuch (Guess)
     * @param lastResult Resultat des vorherigen Guess
     * @param guessID der wie vielte Rateversuch
     * @return nächsten zu vergleichenen Guess
     */
    @Override
    public Guess getNextGuess(Result lastResult, int guessID)
    {
        //compute possibleGuessResult
        // multi threading !!!!!
        getAsyncNextGuess(this.lastGuess, lastResult);
        possibleGuess.clear();
        possibleGuess.addAll(possibleGuessResult);
        possibleGuessResult.clear();


        // sorted list - last one has the highest score
        int posNextGuess = possibleGuess.size() - 1;

        // use 0.47'th guess from list for next guess
        if (possibleGuess.size() > 1) {
            posNextGuess = (int) Math.floor(possibleGuess.size() * 0.47);
            }
        this.lastGuess = possibleGuess.get(posNextGuess);

        possibleGuess.remove(possibleGuess.indexOf(this.lastGuess));
        return this.lastGuess;
    }

    private void getAsyncNextGuess(Guess lastGuess, Result lastResult)
    {
        possibleGuessResult = new ArrayList<Guess>(possibleGuess.size());

        Thread[] threads;
        int startPos, endPos;
        int nbrOfElementsPerSubList = (int) Math.floor(possibleGuess.size() / nbrOfThreads);
        threads = new Thread[nbrOfThreads];

        // Kein Multi-Threading wenn nur noch weniger als 1000 Guesses
        if (possibleGuess.size() < 1000) {
            nbrOfThreads = 1;
        }

        // Thread inizialisieren und starten
        for (int i = 0; i < nbrOfThreads; i ++) {
            // Start und End Pos in der Liste berechnen für die einzelnen Threads
            startPos = i * nbrOfElementsPerSubList;
            if (i < (nbrOfThreads - 1)) {
                endPos = ((i + 1) * nbrOfElementsPerSubList) - 1;
            } else {
                endPos = possibleGuess.size();
            }
            //System.out.println(i + ": " + startPos + " - " + endPos);
            synchronized (this) {
                SolverThreadGetNextGuess myThread = new SolverThreadGetNextGuess(this, lastGuess, lastResult, i);
                List<Guess> tmpList = new ArrayList<Guess>();
                tmpList.addAll(possibleGuess.subList(startPos, endPos));
                myThread.setGuessList(tmpList);
                threads[i] = new Thread(myThread);
                threads[i].start();
            }
        }

        // Warten bis alle Threads fertig sind
        try {
            for (int i = 0; i < nbrOfThreads; i ++) {
                threads[i].join();
            }
        } catch (InterruptedException ex) {
            System.out.println("Error: " + ex.getLocalizedMessage());
        }
    }

    /**
     * Setter-Methode für die einzelnen Threads um ihre Resultat zu speichern
     * @param possibleGuess Liste mit möglichen Guesses
     */
    public synchronized void setListGuessResult(List<Guess> possibleGuess)
    {
        possibleGuessResult.addAll(possibleGuess);
    }

    /**
     * Setter-Methode für die einzelnen Threads um ihre Guess's zu speichern
     * @param newGuess Liste mit den einzufügenden Guesses
     */
    public synchronized void setListGuess(List<Guess> newGuess)
    {
        this.possibleGuess.addAll(newGuess);
    }

    /**
     * Überprüft ob der Solver diese Settings lösen kann ==> Memory !!!!!.
     * Für die Logik wäre es kein Problem...
     * @return true falls lösbar
     */
    public static boolean solvableSettings()
    {
        int numberOfColors = SettingsHandler.getInstance().getSettings().getNumberOfColors();
        boolean allowDuplicateColor = SettingsHandler.getInstance().getSettings().getAllowDuplicateColor();
        int numberOfPegs = SettingsHandler.getInstance().getSettings().getNumberOfPegs();

        boolean res = true;
        int maxGuess = 0;
        if (allowDuplicateColor) {
            maxGuess = (int) Math.pow(numberOfColors, numberOfPegs);
        } else {
            maxGuess = (factorial(numberOfPegs) / factorial(numberOfPegs - numberOfColors));
        }
        if (maxGuess > 1000000000) {
            res = false;
        }
        if (allowDuplicateColor) {
            if (numberOfColors >= 7) {
                if (numberOfPegs >= 9) {
                    res = false;
                }
            } else if (numberOfColors >= 6) {
                if (numberOfPegs >= 10) {
                    res = false;
                }
            }
        }
        return res;
    }
}

